@@ -7,6 +7,7 @@ from .logger import get_caller_name, get_root_logger, log_img_scale
 from .memory import AvoidCUDAOOM, AvoidOOM
 from .misc import find_latest_checkpoint, update_data_root
 from .replace_cfg_vals import replace_cfg_vals
+from .rfnext import rfnext_init_model
 from .setup_env import setup_multi_processes
 from .split_batch import split_batch
 from .util_distribution import build_ddp, build_dp, get_device
@@ -16,5 +17,6 @@ __all__ = [
     'update_data_root', 'setup_multi_processes', 'get_caller_name',
     'log_img_scale', 'compat_cfg', 'split_batch', 'build_ddp', 'build_dp',
     'get_device', 'replace_cfg_vals', 'AvoidOOM', 'AvoidCUDAOOM',
-    'get_max_num_gt_division_factor', 'masked_fill', 'batch_images_to_levels'
+    'get_max_num_gt_division_factor', 'masked_fill', 'batch_images_to_levels',
+    'rfnext_init_model'
 ]
